import { IOverlayBundleData } from '../../interfaces';
import { BaseOverlay, BrandingOverlay, CaptureBase, CharacterEncodingOverlay, FormatOverlay, LabelOverlay, LegacyBrandingOverlay, MetaOverlay, OverlayBundle, OverlayType } from '../../types';
import { Field } from './record';
export declare const BrandingOverlayType: {
    readonly Branding01: OverlayType.Branding01;
    readonly Branding10: OverlayType.Branding10;
    readonly Branding11: OverlayType.Branding11;
};
export type BrandingOverlayType = (typeof BrandingOverlayType)[keyof typeof BrandingOverlayType];
export interface CredentialOverlay<T> {
    bundle?: OCABundle;
    presentationFields?: Field[];
    metaOverlay?: MetaOverlay;
    brandingOverlay?: T;
}
export interface OCABundleType {
    get captureBase(): CaptureBase;
    get metaOverlay(): MetaOverlay | undefined;
    get labelOverlay(): LabelOverlay | undefined;
    get formatOverlay(): FormatOverlay | undefined;
    get characterEncodingOverlay(): CharacterEncodingOverlay | undefined;
    get brandingOverlay(): BrandingOverlay | LegacyBrandingOverlay | undefined;
}
export type OCABundleResolveParams = {
    identifiers: Identifiers;
    language?: string;
};
export type OCABundleResolveDefaultParams = {
    identifiers: Identifiers;
    meta?: Meta;
    language?: string;
};
export type OCABundleResolveAllParams = {
    identifiers: Identifiers;
    attributes?: Array<Field>;
    meta?: Meta;
    language?: string;
};
export type OCABundleResolvePresentationFieldsParams = {
    identifiers: Identifiers;
    attributes: Array<Field>;
    language?: string;
};
export interface OCABundleResolverType {
    resolve(params: OCABundleResolveParams): Promise<OCABundle | undefined>;
    resolveDefaultBundle(params: OCABundleResolveDefaultParams): Promise<OCABundle | undefined>;
    presentationFields(params: OCABundleResolvePresentationFieldsParams): Promise<Field[]>;
    resolveAllBundles(params: OCABundleResolveAllParams): Promise<CredentialOverlay<BaseOverlay | BrandingOverlay | LegacyBrandingOverlay>>;
    getBrandingOverlayType(): BrandingOverlayType;
}
export interface OCABundleResolverOptions {
    language?: string;
    brandingOverlayType?: BrandingOverlayType;
}
export interface Identifiers {
    schemaId?: string;
    credentialDefinitionId?: string;
    templateId?: string;
}
export interface Meta {
    alias?: string;
    credName?: string;
    credConnectionId?: string;
    logo?: string;
}
export declare class OCABundle implements OCABundleType {
    private bundle;
    private options;
    constructor(bundle: OverlayBundle, options?: OCABundleResolverOptions);
    get captureBase(): CaptureBase;
    get characterEncodingOverlay(): CharacterEncodingOverlay | undefined;
    get formatOverlay(): FormatOverlay | undefined;
    get labelOverlay(): LabelOverlay | undefined;
    get metaOverlay(): MetaOverlay | undefined;
    get brandingOverlay(): BrandingOverlay | LegacyBrandingOverlay | undefined;
    buildOverlay(name: string, language: string): MetaOverlay;
    private getOverlay;
}
export declare class DefaultOCABundleResolver implements OCABundleResolverType {
    protected bundles: Record<string, OverlayBundle | string>;
    protected options: OCABundleResolverOptions;
    private _log?;
    constructor(bundlesData?: Record<string, IOverlayBundleData | string>, options?: OCABundleResolverOptions);
    /**
     * Sets the log value.
     * @param value - The new value for the log.
     */
    set log(value: any);
    /**
     * Get the log value.
     */
    get log(): any;
    getBrandingOverlayType(): BrandingOverlayType;
    private getDefaultBundle;
    resolveDefaultBundle(params: OCABundleResolveDefaultParams): Promise<OCABundle | undefined>;
    resolve(params: OCABundleResolveParams): Promise<OCABundle | undefined>;
    presentationFields(params: OCABundleResolvePresentationFieldsParams): Promise<Field[]>;
    resolveAllBundles(params: OCABundleResolveAllParams): Promise<CredentialOverlay<BaseOverlay | BrandingOverlay | LegacyBrandingOverlay>>;
}
