"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultOCABundleResolver = exports.OCABundle = exports.BrandingOverlayType = void 0;
const lodash_startcase_1 = __importDefault(require("lodash.startcase"));
const constants_1 = require("../../constants");
const types_1 = require("../../types");
const color_1 = require("../../utils/color");
const credential_definition_1 = require("../../utils/credential-definition");
exports.BrandingOverlayType = {
    Branding01: types_1.OverlayType.Branding01,
    Branding10: types_1.OverlayType.Branding10,
    Branding11: types_1.OverlayType.Branding11,
};
class OCABundle {
    constructor(bundle, options) {
        var _a, _b;
        this.bundle = bundle;
        this.options = {
            brandingOverlayType: (_a = options === null || options === void 0 ? void 0 : options.brandingOverlayType) !== null && _a !== void 0 ? _a : exports.BrandingOverlayType.Branding10,
            language: (_b = options === null || options === void 0 ? void 0 : options.language) !== null && _b !== void 0 ? _b : constants_1.defaultBundleLanguage,
        };
        // Make bundle overlay type come from options.brandingOverlayType
        this.bundle.overlays.forEach((o) => {
            if (o.type === exports.BrandingOverlayType.Branding10 || o.type === exports.BrandingOverlayType.Branding11) {
                o.type = this.options.brandingOverlayType;
            }
        });
    }
    get captureBase() {
        const overlay = this.bundle.captureBase;
        if (!overlay) {
            throw new Error('Capture Base must be defined');
        }
        return overlay;
    }
    get characterEncodingOverlay() {
        return this.getOverlay(types_1.OverlayType.CharacterEncoding10);
    }
    get formatOverlay() {
        return this.getOverlay(types_1.OverlayType.Format10);
    }
    get labelOverlay() {
        return this.getOverlay(types_1.OverlayType.Label10, this.options.language);
    }
    get metaOverlay() {
        return this.getOverlay(types_1.OverlayType.Meta10, this.options.language);
    }
    get brandingOverlay() {
        var _a;
        return this.getOverlay(((_a = this.options) === null || _a === void 0 ? void 0 : _a.brandingOverlayType) || exports.BrandingOverlayType.Branding10);
    }
    buildOverlay(name, language) {
        return new types_1.MetaOverlay({
            capture_base: '',
            type: types_1.OverlayType.Meta10,
            name,
            language,
            description: '',
            credential_help_text: '',
            credential_support_url: '',
            issuer: '',
            issuer_description: '',
            issuer_url: '',
        });
    }
    getOverlay(type, language) {
        if (type === types_1.OverlayType.CaptureBase10) {
            return this.bundle.captureBase;
        }
        if (language !== undefined) {
            // we want to return branding even if there isn't a bundle for a given language
            const overlay = this.bundle.overlays.find((item) => (item.language === undefined && item.type === type.toString()) ||
                (item.type === type.toString() && item.language === language));
            if (overlay) {
                return overlay;
            }
        }
        return this.bundle.overlays.find((item) => item.type === type.toString());
    }
}
exports.OCABundle = OCABundle;
class DefaultOCABundleResolver {
    constructor(bundlesData = {}, options) {
        var _a, _b, _c;
        this.bundles = {};
        for (const cid in bundlesData) {
            try {
                if (typeof bundlesData[cid] !== 'string') {
                    this.bundles[cid] = new types_1.OverlayBundle(cid, bundlesData[cid]);
                }
                else {
                    this.bundles[cid] = bundlesData[cid];
                }
            }
            catch (error) {
                // might get an error trying to parse javascript's default value
                (_a = this.log) === null || _a === void 0 ? void 0 : _a.error(`Error parsing bundle for ${cid}`, error);
            }
        }
        this.options = {
            brandingOverlayType: (_b = options === null || options === void 0 ? void 0 : options.brandingOverlayType) !== null && _b !== void 0 ? _b : exports.BrandingOverlayType.Branding10,
            language: (_c = options === null || options === void 0 ? void 0 : options.language) !== null && _c !== void 0 ? _c : constants_1.defaultBundleLanguage,
        };
    }
    /**
     * Sets the log value.
     * @param value - The new value for the log.
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    set log(value) {
        this._log = value;
    }
    /**
     * Get the log value.
     */
    get log() {
        return this._log;
    }
    getBrandingOverlayType() {
        var _a;
        return (_a = this.options.brandingOverlayType) !== null && _a !== void 0 ? _a : exports.BrandingOverlayType.Branding10;
    }
    getDefaultBundle(params) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        if (!params.language) {
            params.language = constants_1.defaultBundleLanguage;
        }
        const metaOverlay = {
            capture_base: '',
            type: types_1.OverlayType.Meta10,
            name: (0, lodash_startcase_1.default)((_b = (_a = params.meta) === null || _a === void 0 ? void 0 : _a.credName) !== null && _b !== void 0 ? _b : (0, credential_definition_1.parseCredDefFromId)((_c = params.identifiers) === null || _c === void 0 ? void 0 : _c.credentialDefinitionId, (_d = params.identifiers) === null || _d === void 0 ? void 0 : _d.schemaId)),
            issuer: ((_e = params.meta) === null || _e === void 0 ? void 0 : _e.alias) || ((_f = params.meta) === null || _f === void 0 ? void 0 : _f.credConnectionId) || 'Unknown Contact',
            language: (_g = params.language) !== null && _g !== void 0 ? _g : (_h = this.options) === null || _h === void 0 ? void 0 : _h.language,
            description: '',
            credential_help_text: '',
            credential_support_url: '',
            issuer_description: '',
            issuer_url: '',
        };
        let colorHash = 'default';
        if (metaOverlay === null || metaOverlay === void 0 ? void 0 : metaOverlay.name) {
            colorHash = metaOverlay.name;
        }
        else if (metaOverlay === null || metaOverlay === void 0 ? void 0 : metaOverlay.issuer) {
            colorHash = metaOverlay.issuer;
        }
        const brandingOverlay01 = {
            capture_base: '',
            type: types_1.OverlayType.Branding01,
            background_color: (0, color_1.generateColor)(colorHash),
        };
        const brandingOverlay10 = {
            capture_base: '',
            type: types_1.OverlayType.Branding10,
            primary_background_color: (0, color_1.generateColor)(colorHash),
        };
        const brandingOverlay11 = {
            capture_base: '',
            type: types_1.OverlayType.Branding11,
            primary_background_color: '#FFFFFF',
            secondary_background_color: (0, color_1.generateColor)(colorHash),
        };
        let brandingOverlay = this.getBrandingOverlayType() === exports.BrandingOverlayType.Branding01 ? brandingOverlay01 : brandingOverlay10;
        if (this.getBrandingOverlayType() === exports.BrandingOverlayType.Branding11) {
            brandingOverlay = brandingOverlay11;
        }
        const bundle = new types_1.OverlayBundle((_j = params.identifiers) === null || _j === void 0 ? void 0 : _j.credentialDefinitionId, {
            capture_base: {
                attributes: {},
                classification: '',
                type: types_1.OverlayType.CaptureBase10,
                flagged_attributes: [],
            },
            overlays: [metaOverlay, brandingOverlay],
        });
        return Promise.resolve(new OCABundle(bundle, Object.assign(Object.assign({}, this.options), { language: (_k = params.language) !== null && _k !== void 0 ? _k : this.options.language })));
    }
    resolveDefaultBundle(params) {
        return this.getDefaultBundle(params);
    }
    resolve(params) {
        var _a, _b, _c;
        const language = params.language || constants_1.defaultBundleLanguage;
        for (const item of [
            (_a = params.identifiers) === null || _a === void 0 ? void 0 : _a.credentialDefinitionId,
            (_b = params.identifiers) === null || _b === void 0 ? void 0 : _b.schemaId,
            (_c = params.identifiers) === null || _c === void 0 ? void 0 : _c.templateId,
        ]) {
            if (item && this.bundles[item] !== undefined) {
                let bundle = this.bundles[item];
                // if it is a string, it is a reference/alias to another one bundle
                if (typeof bundle === 'string') {
                    bundle = this.bundles[bundle];
                }
                return Promise.resolve(new OCABundle(bundle, Object.assign(Object.assign({}, this.options), { language: language !== null && language !== void 0 ? language : this.options.language })));
            }
        }
        return Promise.resolve(undefined);
    }
    async presentationFields(params) {
        var _a, _b, _c, _d, _e, _f, _g;
        const bundle = await this.resolve(params);
        let presentationFields = [...params.attributes];
        if ((_a = bundle === null || bundle === void 0 ? void 0 : bundle.captureBase) === null || _a === void 0 ? void 0 : _a.attributes) {
            // if the oca branding has the attributes set, only display those attributes
            const bundleFields = Object.keys(bundle.captureBase.attributes);
            presentationFields = presentationFields.filter((item) => item.name && bundleFields.includes(item.name));
            for (let i = 0; i < presentationFields.length; i++) {
                const presentationField = presentationFields[i];
                const key = presentationField.name || '';
                if (bundle.captureBase.attributes[key]) {
                    presentationField.label = (_b = bundle === null || bundle === void 0 ? void 0 : bundle.labelOverlay) === null || _b === void 0 ? void 0 : _b.attributeLabels[key];
                    presentationField.format = (_c = bundle === null || bundle === void 0 ? void 0 : bundle.formatOverlay) === null || _c === void 0 ? void 0 : _c.attributeFormats[key];
                    presentationField.type = (_e = (_d = bundle === null || bundle === void 0 ? void 0 : bundle.captureBase) === null || _d === void 0 ? void 0 : _d.attributes) === null || _e === void 0 ? void 0 : _e[key];
                    presentationField.encoding = (_g = (_f = bundle === null || bundle === void 0 ? void 0 : bundle.characterEncodingOverlay) === null || _f === void 0 ? void 0 : _f.attributeCharacterEncoding) === null || _g === void 0 ? void 0 : _g[key];
                }
            }
        }
        return presentationFields;
    }
    async resolveAllBundles(params) {
        var _a;
        const [bundle, defaultBundle] = await Promise.all([this.resolve(params), this.resolveDefaultBundle(params)]);
        const fields = params.attributes
            ? await this.presentationFields(Object.assign(Object.assign({}, params), { attributes: params.attributes }))
            : [];
        const overlayBundle = bundle !== null && bundle !== void 0 ? bundle : defaultBundle;
        const metaOverlay = overlayBundle === null || overlayBundle === void 0 ? void 0 : overlayBundle.metaOverlay;
        const brandingOverlay = overlayBundle === null || overlayBundle === void 0 ? void 0 : overlayBundle.brandingOverlay;
        if (brandingOverlay && 'logo' in brandingOverlay && ((_a = params.meta) === null || _a === void 0 ? void 0 : _a.logo)) {
            brandingOverlay.logo = params.meta.logo;
        }
        return { bundle: overlayBundle, presentationFields: fields, metaOverlay, brandingOverlay };
    }
}
exports.DefaultOCABundleResolver = DefaultOCABundleResolver;
